%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright (c) 2023, Amon Lahr, Simon Muntwiler, Antoine Leeman & Fabian Flürenbrock Institute for Dynamic Systems and Control, ETH Zurich.
%
% All rights reserved.
%
% Please see the LICENSE file that has been included as part of this package.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

classdef MPC_TE
    properties
        yalmip_optimizer
    end

    methods
        function obj = MPC_TE(Q,R,N,params)
            % YOUR CODE HERE
            opts = sdpsettings('verbose',1,'solver','quadprog');

            % Create yalmip variables
            U = sdpvar(repmat(params.model.nu,N,1),ones(1,N));
            X = sdpvar(repmat(params.model.nx,N+1,1),ones(1,N+1));

            % Build cost function and constraints
            objective = 0;
            constraints = [];
            for k=1:N+1
                if k <= N
                    % These constraints are for the index in 1, ..., N
                    objective = objective + X{k}' * Q * X{k} + U{k}' * R * U{k};
                    constraints = [constraints, ...
                        X{k+1} == params.model.A * X{k} + params.model.B * U{k}, ...
                        params.constraints.InputMatrix * U{k} <= params.constraints.InputRHS, ...
                    ];
                end

                % These contrains are for 1, ..., N+1
                constraints = [constraints, ...
                    params.constraints.StateMatrix * X{k} <= params.constraints.StateRHS, ...
                ];
            end
            % Terminal constraint
            constraints = [constraints, X{N+1} == zeros(size(X{N+1}))];

            obj.yalmip_optimizer = optimizer(constraints,objective,opts,X{1},{U{1} objective});
        end

        function [u, ctrl_info] = eval(obj,x)
            %% evaluate control action by solving MPC problem, e.g.
            tic;
            [optimizer_out,errorcode] = obj.yalmip_optimizer(x);
            solvetime = toc;
            [u, objective] = optimizer_out{:};

            feasible = true;
            if (errorcode ~= 0)
                feasible = false;
            end

            ctrl_info = struct('ctrl_feas',feasible,'objective',objective,'solvetime',solvetime);
        end
    end
end