%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright (c) 2023, Amon Lahr, Simon Muntwiler, Antoine Leeman & Fabian Flürenbrock Institute for Dynamic Systems and Control, ETH Zurich.
%
% All rights reserved.
%
% Please see the LICENSE file that has been included as part of this package.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

classdef MPC_TE_forces
    properties
        forces_optimizer
    end

    methods
        function obj = MPC_TE_forces(Q,R,N,params)
            % YOUR CODE HERE
            % Create yalmip variables
            U = sdpvar(params.model.nu, N, 'full');
            X = sdpvar(params.model.nx, N+1, 'full');

            % Build cost function and constraints
            objective = 0;
            constraints = [];
            for k=1:N+1
                if k <= N
                    % These constraints are for the index in 1, ..., N
                    objective = objective + X(:,k)' * Q * X(:,k) + U(:,k)' * R * U(:,k);
                    constraints = [constraints, ...
                        X(:,k+1) == params.model.A * X(:,k) + params.model.B * U(:,k), ...
                        params.constraints.InputMatrix * U(:,k) <= params.constraints.InputRHS, ...
                    ];
                end

                % These contrains are for 1, ..., N+1
                constraints = [constraints, ...
                    params.constraints.StateMatrix * X(:,k) <= params.constraints.StateRHS, ...
                ];
            end
            % Terminal constraint
            constraints = [constraints, X(:,N+1) == zeros(size(X(:,N+1)))];

            opts = getOptions('forcesSolver');
            opts.printlevel = 0;
            obj.forces_optimizer = optimizerFORCES(constraints, objective, opts, X(:,1), U(:,1), {'xinit'}, {'u0'});
        end

        function [u, ctrl_info] = eval(obj,x)
            %% evaluate control action by solving MPC problem, e.g.
            [optimizer_out,errorcode,info] = obj.forces_optimizer(x);
            u = optimizer_out;
            objective = info.pobj;
            solvetime = info.solvetime;

            feasible = true;
            if any(errorcode ~= 1)
                feasible = false;
                warning('MPC infeasible');
            end

            ctrl_info = struct('ctrl_feas',feasible,'objective',objective,'solvetime',solvetime);
        end
    end
end